﻿using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;

namespace LastColumnFillDataGrid
{
    /// <summary>
    ///     Adds ability to have the Last Column fill the space in the DataGrid
    ///       (maybe one day star sizing will works for DataGridColumn descendants and we won't need this)
    /// </summary>
    public class ExtendedDataGrid: DataGrid
    {
        public ExtendedDataGrid()
        {
        }

        #region Overridden methods
        /// <summary>
        ///     Override of base ArrangeOverride to use additional properties
        /// </summary>
        /// <param name="finalSize"></param>
        /// <returns></returns>
        protected override Size ArrangeOverride(Size finalSize)
        {
            Size size = base.ArrangeOverride(finalSize);

            if (LastColumnFill)
                AdjustLastColumnWidth(finalSize);

            return size;
        }
        #endregion

        #region Dependency Properties
        public static readonly DependencyProperty LastColumnFillProperty =
           DependencyProperty.Register("LastColumnFill", typeof(bool), typeof(ExtendedDataGrid), null);

        public bool LastColumnFill
        {
            get { return (bool) GetValue(LastColumnFillProperty); }
            set { SetValue(LastColumnFillProperty, value); }
        }
        #endregion

        #region Private methods
        /// <summary>
        ///     Adjusts the last column to make it fill the remaining area of the DataGrid
        /// </summary>
        /// <param name="finalSize"></param>
        private void AdjustLastColumnWidth(Size finalSize)
        {
            // get the Vertical ScrollBar
            ScrollBar scrollBar = this.GetTemplateChild("VerticalScrollbar") as ScrollBar;

            // compute the width to allow for the scrollbar based on it's visibility.
            double scrollBarWidthAllowance = (scrollBar != null && scrollBar.Visibility == Visibility.Visible) ? scrollBar.ActualWidth + 2 : 2;

            // compute the width of all the columns excluding the last one
            var widthOfAllButLastColumn = this.Columns
                                            .TakeWhile(c => c != Columns.Last() &&
                                                            c.Visibility == Visibility.Visible)
                                            .Sum(c => c.ActualWidth);

            // set the last column width    
            this.Columns.Last().Width = new DataGridLength(finalSize.Width - widthOfAllButLastColumn - scrollBarWidthAllowance);
        }
        #endregion
    }
}
